/******************************************************************************
 * Boot Loader v1.0                                                           *
 * di Valerio Di Gregorio                                                     *
 *                                                                            *
 * Boot Loader v1.0 carica da shell di FreeDOS il "Nucleo di Calcolatori      *
 * Elettronici v1.0". Questo progetto  conforme alle "Multiboot              *
 * Specification version 0.6.95".                                             *
 ******************************************************************************/

#ifndef ___COFF16_H_
#define ___COFF16_H_

#include <stdio.h>
#include "coff.h"

typedef char byte;

struct coff_exe_go32 {
	FILE* obj;                       /* Oggetto eseguibile COFF */
	FILHDR filhdr;                   /* File header */
	AOUTHDR opthdr;                  /* Optional header */
	unsigned long soff;              /* Offset di inizio dell'oggetto COFF (permette di saltare le strutture iniziali del DOS) */
	SCNHDR* seghdr;                  /* Section headers */
};

#define ExeCoff_go32 struct coff_exe_go32

typedef union{unsigned long ul; byte* v;} pointer;

/*
 * ATTENZIONE: la seguente struttura presenta campi di tipo union. L'uso di 
 * questi campi  dovuto alla necessit di avere solo campi di 4 byte (i 
 * puntatori in modalit reale occupano 2 byte). Poich questa struttura
 * verr passata alla parte a 32 bit la stessa definizione individuerebbe
 * una rappresentazione in memoria del tutto diversa. Questa soluzione dovrebbe
 * essere solo provvisoria e sostituita con pi elegante definizione in 
 * assembler.
 */
struct coff_img_go32 {
    unsigned long n;                /* Numero di segmenti */
    unsigned long ds;               /* Selettore del segmento in cui  caricata l'immagine (serve per determinare gli indirizzi lineari) */  
	
    union {
       unsigned long ul;
       unsigned long* v;
    } size;                         /* Dimensione dei segmenti */
	
    union {
       unsigned long ul;
       unsigned long* v;
    } vaddr;                        /* Indirizzi lineari in cui  previsto il caricamento dell'eseguibile */
	
    union {
       unsigned long ul;
       pointer* v;
    } segment;                      /* Puntatori ai buffer di memoria contenenti i segmenti */   	
	
    union {
       unsigned long ul;
       void* v;
    } entry_point;                  /* Entry point dell'eseguibile */
};

#define ImgCoff_go32 struct coff_img_go32

extern unsigned long get_ds();                       /* Restituisce il valore del registro DS */
int initialize_exe(FILE*, ExeCoff_go32*);            /* Inizializza la struttura ExeCoff_go32 */
void destroy_exe(ExeCoff_go32*);                     /* Distrugge la struttura ExeCoff_go32 */
void initialize_img(ImgCoff_go32*, unsigned long);   /* Inizializza la struttura ImgCoff_go32 */
void destroy_img(ImgCoff_go32*);                     /* Distrugge la struttura ImgCoff_go32 */
void load_coff(char*, ImgCoff_go32*);                /* Carica in memoria un oggetto COFF e ne restituisce l'immagine */
unsigned long vaddr(unsigned int, unsigned int);   /* Converte un indirizzo selettore:offset in un indirizzo lineare */

#endif /* ___COFF16_H_ */
