;SVGA di Alessandro Furlanetto, Federico Bianucci
;Prima di abilitare la modalita' protetta si abilita la modalita' grafica.
;In particolare si e' scelta la modalita' SVGA 107 hex.
;Le int 10h con AX = 4F01 e AX = 1130h servono ad ottenere informazioni sulla modalita' grafica attiva.
;Queste informazioni vengono poi passate al sistema operativo attraverso la memoria (indirizzi 0x9FFF0-0x9FFFC).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

segment _DATA private class=DATA

buf:
times 64 dd 00000000h
global _lfb
_lfb:
_lfb_L: 
dw	0000h
_lfb_H:
dw	0000h
global _lfb_w
_lfb_w:
dw	0000h
global _lfb_h
_lfb_h:
dw	0000h

segment _TEXT public class=CODE use16

global _textinit
_textinit:

	push bp
	mov bp, sp
        
	push ax
        push bx
        push cx
        push dx
        push ds
        push es
        push bp

txtmode:
	mov bx, 0x9FFF  ;BX     |A000h|
        mov ds, bx      ;DS     |A000h|
	mov bx, 0
	mov [ds:0], bx
	mov [ds:2], bx

; Indichiamo al nucleo che txtmodeamo in modalità testo 
; settando a zero l'indirizzo del linear frame buffer, 
; che altrimenti sarebbe utilizzato dalle funzioni grafiche

        pop bp
        pop es
        pop ds
        pop dx
        pop cx
        pop bx
        pop ax

	mov sp, bp
	pop bp
	ret


global _vgainit
_vgainit:

	push bp 
	mov bp, sp

        push ax
        push bx
        push cx
        push dx
	push di
        push ds
        push es
        push bp

	mov ax, 0x4F00	; verifica compatibilità
	int 0x10;
	cmp ax,0x004F	; verifica successo
	jne txtmode	; (spaghetti style!)

; La int 10h con AX = 4F01h serve ad ottenere informazioni sulla modalità video specificata.
; In particolare, ci occorre per procurarci l'indirizzo del LFB e la risoluzione.

	mov ax, SEG buf
	mov es, ax
	mov di, buf
	mov ax,0x4F01	; funzione raccogli informazioni
	mov cx,0x4107	; modalità attuale
	int 0x10 
	cmp ax,0x004F	; verifica successo
	jne txtmode	; (spaghetti style!)
			; [es:di] |descrittore modalità|
	
	mov cx,di	; cx |offset descrittore modalità|
	mov bx,0x9FFF		 
	mov ds,bx	; ds |selettore svga_info|

	add di,0x28
	mov ax,[es:di]
	mov [ds:0x0],ax	;9FFF0	|selettore indirizzo di base del LFB|
	mov [es:_lfb_L],ax

	mov di,cx
	add di,0x2A
	mov ax,[es:di]
	mov [ds:0x2],ax	;9FFF2	|offset indirizzo di base del LFB|
	mov [es:_lfb_H],ax

; l'indirizzo a 32 bit ora contenuto nella locazione 9FFF0 sarà utilizzato
; dalle funzioni della sezione MONITOR VIRTUALI nel file io.cpp
; e dalla funzione mappa_in_mem_virtuale nel file sistema.cpp
; mediante l'alias svga_info->lfb_addr 

	mov di,cx
	add di,0x12
	mov ax,[es:di]
	mov [ds:0xA],ax	;9FFFA	|width_in_pixel|
	mov [es:_lfb_w],ax

	mov di,cx
	add di,0x14
	mov ax,[es:di]
	mov [ds:0xC],ax	;9FFFC	|height_in_pixel|	
	mov [es:_lfb_h],ax


;L'int 10h con AX = 1130h serve ad ottenere informazioni sulla mappa dei caratteri.
  
	mov ah, 0x11	;AH     |11h|
        mov al, 0x30	;AL     |30h|
        mov bh, 0x06	;BH     |6|
        int 10h         ;ES     |selettore indirizzo di base dei caratteri|
                        ;BP     |offset indirizzo di base dei caratteri|
                        ;CX     |byte per carattere|
        mov ax, 0x9FFF  ;AX     |A000h|
        mov ds, ax      ;DS     |A000h|
        mov [ds:0x4], bp
        mov [ds:0x6], es
        mov [ds:0x8], cx

; Le informazioni ora contenute in 9FFF4 e seguenti saranno utilizzate
; dalla funzione _write_gc nel file io.s
	
	mov ax,0x4F08	; funzione raccogli informazioni
	mov bx,0x0800	; modalità attuale
	int 0x10 
	cmp ax,0x004F	; verifica successo
	;jne txtmode	; (spaghetti style!)
			; [es:di] |descrittore modalità|
	mov ax, 0x4F02	; funzione cambio modalità
	mov bx, 0x0107	; modalità SVGA 1280x1024x256
	or bx,  0x4000	; attiva Linear Frame Buffer
	int 0x10 ; 
	cmp ax,0x004F	; verifica successo
	jne txtmode	; (spaghetti style!)

        pop bp
        pop es
        pop ds
	pop di
        pop dx
        pop cx
        pop bx
        pop ax

	mov sp, bp
	pop bp
	ret

global _ch_getmaxline
_ch_getmaxline:
	push bp
	mov bp, sp
        push ds

        mov ax, 0x9FFF  ;AX     |A000h|
        mov ds, ax      ;DS     |A000h|
        mov ax, [ds:0x8]

        pop ds
	mov sp, bp
        pop bp
        ret

global _ch_getline
_ch_getline:
        push bp
        mov bp, sp
	push ds
	push es
	push bx
	push cx
	push dx

        mov ax, 0x9FFF  ;AX     |A000h|
        mov ds, ax      ;DS     |A000h|

	mov ax, [ds:0x6]
	mov es, ax
	mov bx, [ds:0x4]
	mov ax, [ds:0x8]
	mul word [bp+4]
	add ax, [bp+6]
	add bx, ax
	mov ax, [es:bx]
	
	pop dx
	pop cx
	pop bx
	pop es
	pop ds
	mov sp, bp
	pop bp
	ret

global _vgaclose
_vgaclose:

	push bp
	mov bp, sp

        push ax
        push bx
        push cx
        push dx
        push ds
        push es
        push bp

	mov ax, 0x0003 	; funzione per tornare in modalità testo
	int 0x10

	mov ax,0x0200 	; funzione riposizionamento del cursore
	mov dl,0 	; il cursore viene posizionato all'inizio
	mov dh,0 	; della prima riga
	int 0x10

        pop bp
        pop es
        pop ds
        pop dx
        pop cx
        pop bx
        pop ax

	mov sp, bp
	pop bp
	ret

global _textclose
_textclose:

	push bp
	mov bp, sp

        push ax
        push bx
        push cx
        push dx
        push ds
        push es
        push bp

	mov ax, 0x0003 	; funzione per tornare in modalità testo
	int 0x10

	mov ax,0x0200 	; funzione riposizionamento del cursore
	mov dl,0 	; il cursore viene posizionato all'inizio
	mov dh,0 	; della prima riga
	int 0x10

        pop bp
        pop es
        pop ds
        pop dx
        pop cx
        pop bx
        pop ax

	mov sp, bp
	pop bp
	ret
